import { Stack, StackProps } from "aws-cdk-lib";
import {
  AuthorizationType,
  CognitoUserPoolsAuthorizer,
  Cors,
  LambdaIntegration,
  MethodOptions,
  ResourceOptions,
  RestApi,
} from "aws-cdk-lib/aws-apigateway";
import { IUserPool } from "aws-cdk-lib/aws-cognito";
import { AuthorizationToken } from "aws-cdk-lib/aws-ecr";
import { Construct } from "constructs";

interface ApiStackProps extends StackProps {
  contactsLambdaIntegration: LambdaIntegration;
  userPool: IUserPool;
}

export class ApiStack extends Stack {
  constructor(scope: Construct, id: string, props: ApiStackProps) {
    super(scope, id, props);

    const api = new RestApi(this, "ContactsApi");

    const authorizer = new CognitoUserPoolsAuthorizer(
      this,
      "ContactAppApiAuthorizer",
      {
        cognitoUserPools: [props.userPool], //unosimo user pool
        identitySource: "method.request.header.Authorization", //location of the authorization header
      }
    );
    authorizer._attachToApi(api); //moramo da dodamo api, jer authorizer ne moze da postoji bez api-a

    const optionsWithAuth: MethodOptions = {
      //opcije za autorizaciju
      authorizationType: AuthorizationType.COGNITO, //tip autorizacije
      authorizer: {
        //dodamo authorizer-a kojeg so napravili gore
        authorizerId: authorizer.authorizerId,
      },
    };

    const optionsWithCors: ResourceOptions = {
      //Opcije za CORS
      defaultCorsPreflightOptions: {
        allowOrigins: Cors.ALL_ORIGINS,
        allowMethods: Cors.ALL_METHODS,
      },
    }; //Ovim CORS opcijama dozvoljavamo serveru da handluje requeste iz web browsera

    const contactsResource = api.root.addResource("contacts", optionsWithCors);

    contactsResource.addMethod(
      "GET",
      props.contactsLambdaIntegration,
      optionsWithAuth
    );
    contactsResource.addMethod(
      "POST",
      props.contactsLambdaIntegration,
      optionsWithAuth
    );
    contactsResource.addMethod(
      "PUT",
      props.contactsLambdaIntegration,
      optionsWithAuth
    );
    contactsResource.addMethod(
      "DELETE",
      props.contactsLambdaIntegration,
      optionsWithAuth
    );
  }
}
