import { CfnOutput, Stack, StackProps } from "aws-cdk-lib";
import {
  CfnIdentityPool,
  CfnIdentityPoolRoleAttachment,
  CfnUserPoolGroup,
  UserPool,
  UserPoolClient,
} from "aws-cdk-lib/aws-cognito";
import {
  Effect,
  FederatedPrincipal,
  PolicyStatement,
  Role,
} from "aws-cdk-lib/aws-iam";
import { Construct } from "constructs";

export class AuthStack extends Stack {
  public userPool: UserPool;
  private userPoolClient: UserPoolClient;
  private identityPool: CfnIdentityPool;
  private authenticatedRole: Role;
  private unAuthenticatedRole: Role;
  private adminRole: Role;

  constructor(scope: Construct, id: string, props?: StackProps) {
    super(scope, id, props);

    this.createUserPool();
    this.createUserPoolClient();
    this.createIdentityPool();
    this.createRoles();
    this.attachRoles();
  }

  //Metoda za Kreiranje userPool-a
  private createUserPool() {
    this.userPool = new UserPool(this, "ContactAppUserPool", {
      selfSignUpEnabled: true,
      signInAliases: {
        username: false,
        email: true,
      },
    });
    new CfnOutput(this, "ContactAppUserPoolId", {
      value: this.userPool.userPoolId,
    });
  }

  //User pool ne bi trebalo da postoji bez user pool client-a
  private createUserPoolClient() {
    this.userPoolClient = this.userPool.addClient("ContactAppUserPoolClient", {
      authFlows: {
        adminUserPassword: true,
        custom: true,
        userPassword: true,
        userSrp: true,
      },
    });
    new CfnOutput(this, "ContactAppUserPoolClientId", {
      value: this.userPoolClient.userPoolClientId,
    });
  }

  //Pravimo identity pool za privremene usere.
  private createIdentityPool() {
    this.identityPool = new CfnIdentityPool(this, "ContactAppIdentityPool", {
      allowUnauthenticatedIdentities: true, //Dozvoljavamo ne autorizovane usere
      cognitoIdentityProviders: [
        //Provajderi za autorizaciju. Koristimo nas cognito, ali mozemo i neke druge.
        {
          clientId: this.userPoolClient.userPoolClientId,
          providerName: this.userPool.userPoolProviderName,
        },
      ],
    });
    new CfnOutput(this, "ContactAppIdentityPoolId", {
      value: this.identityPool.ref, //treba nam ovaj value van ovog stack-a korisitmo cfnoutput
    });
  }

  /////// dodavanja IAM Roles
  private createRoles() {
    this.authenticatedRole = new Role(this, "CognitoDefaultAuthenticatedRole", {
      assumedBy: new FederatedPrincipal(
        "cognito-identity.amazonaws.com",
        {
          StringEquals: {
            "cognito-identity.amazonaws.com:aud": this.identityPool.ref,
          },
          "ForAnyValue:StringLike": {
            "cognito-identity.amazonaws.com:amr": "authenticated",
          },
        },
        "sts:AssumeRoleWithWebIdentity"
      ),
    });
    this.unAuthenticatedRole = new Role(
      this,
      "CognitoDefaultUnauthenticatedRole",
      {
        assumedBy: new FederatedPrincipal(
          "cognito-identity.amazonaws.com",
          {
            StringEquals: {
              "cognito-identity.amazonaws.com:aud": this.identityPool.ref,
            },
            "ForAnyValue:StringLike": {
              "cognito-identity.amazonaws.com:amr": "unauthenticated",
            },
          },
          "sts:AssumeRoleWithWebIdentity"
        ),
      }
    );
  }
  //attachujemo role
  private attachRoles() {
    new CfnIdentityPoolRoleAttachment(this, "RolesAttachment", {
      identityPoolId: this.identityPool.ref, //identity pool ref zato sto nema id
      roles: {
        authenticated: this.authenticatedRole.roleArn,
        unauthenticated: this.unAuthenticatedRole.roleArn,
      },
      roleMappings: {
        //Na osnovu tokena ce da razlikuje role
        adminsMapping: {
          type: "Token",
          ambiguousRoleResolution: "AuthenticatedRole",
          identityProvider: `${this.userPool.userPoolProviderName}:${this.userPoolClient.userPoolClientId}`,
        },
      },
    });
  }
  ///////////////////
}
